From 5ab2021faf64a566617bc415a2ef333aab239581 Mon Sep 17 00:00:00 2001
From: Benjamin Gerard <benjihan@users.sourceforge.net>
Date: Thu, 19 Feb 2009 03:19:20 +0100
Subject: [PATCH] Add a file extension manager to libvlccore. This manager allows to register file extensions associate with categories (currently audio video play-list and subtitle). It is thread safe. For now it is initialized with the old hard-coded vlc extension list. In the future modules (mainly demuxer but not necessary) should register their file extensions at startup. For this to work properly the module cache (and the module struct) probably need to be modified so that file extensions registered by a module appear in the cache. This would be my next move.

---
 include/vlc_fileext.h                |  116 ++++++++
 include/vlc_interface.h              |   26 +--
 modules/gui/qt4/dialogs_provider.hpp |   51 ++--
 modules/gui/skins2/src/dialogs.cpp   |   28 ++-
 src/Makefile.am                      |    2 +
 src/libvlc.c                         |    6 +
 src/libvlccore.sym                   |    5 +
 src/misc/fileext.c                   |  503 ++++++++++++++++++++++++++++++++++
 8 files changed, 689 insertions(+), 48 deletions(-)
 create mode 100644 include/vlc_fileext.h
 create mode 100644 src/misc/fileext.c

diff --git a/include/vlc_fileext.h b/include/vlc_fileext.h
new file mode 100644
index 0000000..857d704
--- /dev/null
+++ b/include/vlc_fileext.h
@@ -0,0 +1,116 @@
+/*****************************************************************************
+ * vlc_fileext.h: file extension for media
+ *****************************************************************************
+ * Copyright (C) 2004-2009 the VideoLAN team
+ * $Id$
+ *
+ * Authors: Benjamin Gerard <benjihan -4t- users.sourceforge -d0t- net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifndef VLC_FILEEXT_H
+# define VLC_FILEEXT_H
+
+/**
+ * \file vlc_fileext.h
+ * This file defines functions and structures for handling file extensions
+ */
+
+/**
+ * \name     file extension types
+ * \{
+ * \notice   These defines are bit mask.
+ * \warning  DO NOT CHANGE ORDER without changing names table in fileext.c
+ */
+#define FILEEXT_UNKNOWN   0   /**< file extension has no type         */
+#define FILEEXT_AUDIO     1   /**< audio file extension mask.         */
+#define FILEEXT_VIDEO     2   /**< video file extension mask.         */
+#define FILEEXT_PLAYLIST  4   /**< playlist file extension mask.      */
+#define FILEEXT_SUBTITLE  8   /**< subtitle file extension mask.      */
+/** all media file extension mask (consists on audio+video+playlist). */
+#define FILEEXT_MEDIA     (FILEEXT_AUDIO|FILEEXT_VIDEO|FILEEXT_PLAYLIST)
+/** all defined file extension mask (consist on media+subtitle).      */
+#define FILEEXT_ALL_KNOWN (FILEEXT_MEDIA|FILEEXT_SUBTITLE)
+#define FILEEXT_ALL       16  /**< all (ALL_KNOWN+UNKNOW).            */
+/** invalid extension.                                                */
+#define FILEEXT_ERROR     -1
+/**
+ * \}
+ */
+
+/**
+ * Initialise extension manager.
+ *
+ * \retval  VLC_SUCCESS  on success
+ * \retval  VLC_EGENERIC on failure
+ */
+LIBVLC_EXTERN int fileext_Init(void);
+
+/**
+ * Cleanup extension manager.
+ *
+ */
+LIBVLC_EXTERN void fileext_Cleanup(void);
+
+/**
+ * Get name of extension type(s).
+ *
+ * \param   types  extension type mask
+ * \retval  0      an invalid type
+ * \retval  other  a semicolon ';' separated list of extension type
+ */
+VLC_EXPORT( const char *, fileext_TypeName, ( int types ));
+
+/**
+ * Get type of extension.
+ *
+ * \param   psz_ext  single extension ("ext" or ".ext" or "*.ext")
+ * \return  FILEEXT_???
+ */
+VLC_EXPORT( int, fileext_TypeOf, ( const char * psz_ext ) );
+
+/**
+ * Get extension list of given types.
+ *
+ * \param   types  extension type mask
+ * \retval  0      an invalid type
+ * \retval  ""     an empty list
+ * \retval  other  a list of extension (e.g "*.ext1;*.ext2")
+ */
+VLC_EXPORT( char *, fileext_Get, ( int types ) );
+
+/**
+ * Add a list of extension of given type.
+ *
+ * \param   psz_exts  a semicolon ';' separated list of extension
+ * \param   types     extension type mask (for a single type)
+ * \retval  VLC_EGENERIC  on failure
+ * \retval  VLC_SUCCESS   on success
+ */
+VLC_EXPORT( int , fileext_Add, ( const char * psz_exts, int types ) );
+
+/**
+ * Delete (remove) a list of extension.
+ *
+ * \param   psz_exts  a semicolon ';' separated list of extension
+ * \param   types     extension type mask (for a single type)
+ * \retval  VLC_EGENERIC  on failure
+ * \retval  VLC_SUCCESS   on success
+ */
+VLC_EXPORT( int , fileext_Del, ( const char * psz_exts, int types ) );
+
+#endif
+
diff --git a/include/vlc_interface.h b/include/vlc_interface.h
index 7dde2a5..eb2720d 100644
--- a/include/vlc_interface.h
+++ b/include/vlc_interface.h
@@ -171,23 +171,15 @@ typedef enum vlc_dialog {
 /* Useful text messages shared by interfaces */
 #define INTF_ABOUT_MSG LICENSE_MSG
 
-#define EXTENSIONS_AUDIO "*.a52;*.aac;*.ac3;*.dts;*.flac;*.m4a;*.m4p;*.mka;" \
-                         "*.mod;*.mp1;*.mp2;*.mp3;*.oga;*.ogg;*.oma;*.spx;" \
-                         "*.wav;*.wma;*.xm"
-
-#define EXTENSIONS_VIDEO "*.asf;*.avi;*.divx;*.dv;*.flv;*.gxf;*.iso;*.m1v;*.m2v;" \
-                         "*.m2t;*.m2ts;*.m4v;*.mkv;*.mov;*.mp2;*.mp4;*.mpeg;*.mpeg1;" \
-                         "*.mpeg2;*.mpeg4;*.mpg;*.mts;*.mxf;*.nuv;" \
-                         "*.ogg;*.ogm;*.ogv;*.ogx;*.ps;" \
-                         "*.rec;*.rm;*.rmvb;*.ts;*.vob;*.wmv;"
-
-#define EXTENSIONS_PLAYLIST "*.asx;*.b4s;*.m3u;*.pls;*.vlc;*.xspf"
-
-#define EXTENSIONS_MEDIA EXTENSIONS_VIDEO ";" EXTENSIONS_AUDIO ";" \
-                          EXTENSIONS_PLAYLIST
-
-#define EXTENSIONS_SUBTITLE "*.cdg;*.idx;*.srt;*.sub;*.utf;*.ass;*.ssa;*.aqt;" \
-                            "*.jss;*.psb;*.rt;*.smi"
+  /* XXX: dynamic file extension:
+     returns MALLOCated strings caller should FREE them !
+  */
+# include <vlc_fileext.h>
+# define _EXTENSIONS_AUDIO    fileext_Get( FILEEXT_AUDIO )
+# define _EXTENSIONS_VIDEO    fileext_Get( FILEEXT_VIDEO )
+# define _EXTENSIONS_PLAYLIST fileext_Get( FILEEXT_PLAYLIST )
+# define _EXTENSIONS_MEDIA    fileext_Get( FILEEXT_MEDIA )
+# define _EXTENSIONS_SUBTITLE fileext_Get( FILEEXT_SUBTITLE )
 
 /** \defgroup vlc_interaction Interaction
  * \ingroup vlc_interface
diff --git a/modules/gui/qt4/dialogs_provider.hpp b/modules/gui/qt4/dialogs_provider.hpp
index c08a6a5..391765d 100644
--- a/modules/gui/qt4/dialogs_provider.hpp
+++ b/modules/gui/qt4/dialogs_provider.hpp
@@ -35,34 +35,31 @@
 
 #include <QObject>
 
-#define ADD_FILTER_MEDIA( string )     \
-    string += qtr( "Media Files" );    \
-    string += " ( ";                   \
-    string += EXTENSIONS_MEDIA;        \
-    string += ");;";
-#define ADD_FILTER_VIDEO( string )     \
-    string += qtr( "Video Files" );    \
-    string += " ( ";                   \
-    string += EXTENSIONS_VIDEO;        \
-    string += ");;";
-#define ADD_FILTER_AUDIO( string )     \
-    string += qtr( "Audio Files" );    \
-    string += " ( ";                   \
-    string += EXTENSIONS_AUDIO;        \
-    string += ");;";
-#define ADD_FILTER_PLAYLIST( string )  \
-    string += qtr( "Playlist Files" ); \
-    string += " ( ";                   \
-    string += EXTENSIONS_PLAYLIST;     \
-    string += ");;";
-#define ADD_FILTER_SUBTITLE( string )  \
-    string += qtr( "Subtitles Files" );\
-    string += " ( ";                   \
-    string += EXTENSIONS_SUBTITLE;     \
-    string += ");;";
+#define ADD_FILTER_ANY( string, type, label )	\
+    if (1) {					\
+	char * ext = _EXTENSIONS_##type;	\
+	string += qtr( label );			\
+	string += " ( ";			\
+	string += ext;				\
+	string += ");;";			\
+	free( ext );				\
+    } else
+
+#define ADD_FILTER_MEDIA( string )\
+    ADD_FILTER_ANY( string,MEDIA,"Media Files" )
+#define ADD_FILTER_VIDEO( string )\
+    ADD_FILTER_ANY( string,VIDEO,"Video Files" )
+#define ADD_FILTER_AUDIO( string )\
+    ADD_FILTER_ANY( string,AUDIO,"Audio Files" )
+#define ADD_FILTER_PLAYLIST( string )\
+    ADD_FILTER_ANY( string,PLAYLIST,"Playlist Files" )
+#define ADD_FILTER_SUBTITLE( string )\
+    ADD_FILTER_ANY( string,SUBTITLE,"Subtitles Files" )
 #define ADD_FILTER_ALL( string )       \
-    string += qtr( "All Files" );      \
-    string += " (*)";
+    if (1) {			       \
+	string += qtr( "All Files" );  \
+	string += " (*)";	       \
+    } else
 
 enum {
     EXT_FILTER_MEDIA     =  0x01,
diff --git a/modules/gui/skins2/src/dialogs.cpp b/modules/gui/skins2/src/dialogs.cpp
index bf4b541..2cfd17f 100644
--- a/modules/gui/skins2/src/dialogs.cpp
+++ b/modules/gui/skins2/src/dialogs.cpp
@@ -223,10 +223,30 @@ void Dialogs::showChangeSkin()
 
 void Dialogs::showPlaylistLoad()
 {
-    showFileGeneric( _("Open playlist"),
-                     _("Playlist Files|"EXTENSIONS_PLAYLIST"|"
-                       "All Files|*"),
-                     showPlaylistLoadCB, kOPEN );
+   char * list;
+   
+   const char * psz_playlist = _("Playlist Files");
+   const char * psz_allfiles = _("All Files");
+   char * psz_pl_exts  = _EXTENSIONS_PLAYLIST;
+
+   int len =
+      strlen (psz_playlist) + strlen(psz_allfiles) + strlen(psz_pl_exts) + 16;
+
+   list = new char [len];
+   if( list )
+   {
+      sprintf(list,"%s|%s|%s|*",psz_playlist,psz_pl_exts,psz_allfiles);
+      showFileGeneric( _("Open playlist"),
+		       list,
+		       showPlaylistLoadCB, kOPEN );
+      delete [] list;
+   }
+   free (psz_pl_exts);
+
+//   showFileGeneric( _("Open playlist"),
+// 		   _("Playlist Files|"EXTENSIONS_PLAYLIST"|"
+// 		     "All Files|*"),
+// 		   showPlaylistLoadCB, kOPEN );
 }
 
 
diff --git a/src/Makefile.am b/src/Makefile.am
index 061e247..b90d4c1 100644
--- a/src/Makefile.am
+++ b/src/Makefile.am
@@ -55,6 +55,7 @@ pluginsinclude_HEADERS = \
 	../include/vlc_es.h \
 	../include/vlc_es_out.h \
 	../include/vlc_events.h \
+	../include/vlc_fileext.h \
 	../include/vlc_filter.h \
 	../include/vlc_gcrypt.h \
 	../include/vlc_httpd.h \
@@ -356,6 +357,7 @@ SOURCES_libvlc_common = \
 	misc/mtime.c \
 	misc/block.c \
 	misc/es_format.c \
+	misc/fileext.c \
 	modules/modules.h \
 	modules/modules.c \
 	modules/cache.c \
diff --git a/src/libvlc.c b/src/libvlc.c
index fb7aa52..e873fbc 100644
--- a/src/libvlc.c
+++ b/src/libvlc.c
@@ -321,6 +321,9 @@ int libvlc_InternalInit( libvlc_int_t *p_libvlc, int i_argc,
      */
     LoadMessages ();
 
+    /* Initialize file extension manager */
+    fileext_Init();
+
     /* Initialize the module bank and load the configuration of the
      * main module. We need to do this at this stage to be able to display
      * a short help if required by the user. (short help == main module
@@ -1111,6 +1114,9 @@ void libvlc_InternalCleanup( libvlc_int_t *p_libvlc )
     /* Free module bank. It is refcounted, so we call this each time  */
     module_EndBank( p_libvlc, true );
 
+    /* Cleanup file extension manager */
+    fileext_Cleanup();
+
     FREENULL( priv->psz_configfile );
     var_DelCallback( p_libvlc, "key-pressed", vlc_key_to_action,
                      p_libvlc->p_hotkeys );
diff --git a/src/libvlccore.sym b/src/libvlccore.sym
index 45497fc..e75dcc8 100644
--- a/src/libvlccore.sym
+++ b/src/libvlccore.sym
@@ -108,6 +108,11 @@ EnsureUTF8
 es_format_Clean
 es_format_Copy
 es_format_Init
+fileext_TypeName
+fileext_TypeOf
+fileext_Get
+fileext_Add
+fileext_Del
 filename_sanitize
 filter_chain_AppendFilter
 filter_chain_AppendFromString
diff --git a/src/misc/fileext.c b/src/misc/fileext.c
new file mode 100644
index 0000000..77c8202
--- /dev/null
+++ b/src/misc/fileext.c
@@ -0,0 +1,503 @@
+/*****************************************************************************
+ * vlc_fileext.c: file extension for media
+ *****************************************************************************
+ * Copyright (C) 2004-2009 the VideoLAN team
+ * $Id$
+ *
+ * Authors: Benjamin Gerard <benjihan -4t- users.sourceforge -d0t- net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <ctype.h>
+#include <string.h>
+#include <stdlib.h>
+
+# include <vlc_common.h>
+# include <vlc_threads.h>
+# include <vlc_fileext.h>
+
+/** Predefined audio file extensions. */
+#define EXTENSIONS_AUDIO_STATIC						\
+  "*.a52;*.aac;*.ac3;*.dts;*.flac;*.m4a;*.m4p;*.mka;"			\
+  "*.mod;*.mp1;*.mp2;*.mp3;*.oga;*.ogg;*.oma;*.spx;"			\
+  "*.wav;*.wma;*.xm"
+
+/* Predefined video file extensions. */
+#define EXTENSIONS_VIDEO_STATIC						\
+  "*.asf;*.avi;*.divx;*.dv;*.flv;*.gxf;*.iso;*.m1v;*.m2v;"		\
+  "*.m2t;*.m2ts;*.m4v;*.mkv;*.mov;*.mp2;*.mp4;*.mpeg;*.mpeg1;"		\
+  "*.mpeg2;*.mpeg4;*.mpg;*.mts;*.mxf;*.nuv;"				\
+  "*.ogg;*.ogm;*.ogv;*.ogx;"						\
+  "*.ps;*.rm;*.rmvb;*.ts;*.vob;*.wmv;"
+
+/* Predefined playlist file extensions. */
+#define EXTENSIONS_PLAYLIST_STATIC		\
+   "*.asx;*.b4s;*.m3u;*.pls;*.vlc;*.xspf"
+
+/* Predefined media file extensions. */
+#define EXTENSIONS_MEDIA_STATIC						\
+  EXTENSIONS_VIDEO_STATIC ";"						\
+  EXTENSIONS_AUDIO_STATIC ";"						\
+  EXTENSIONS_PLAYLIST_STATIC
+
+/* Predefined subtitle file extension. */
+#define EXTENSIONS_SUBTITLE_STATIC					\
+  "*.cdg;*.idx;*.srt;*.sub;*.utf;*.ass;*.ssa;*.aqt;*.jss;*.psb;*.rt;*.smi"
+
+/** maximum bare file extension length (w/o '*.') */
+#define FILEEXT_LEN_MAX 5
+
+typedef struct fileext_s {
+   char psz_ext[8];		/**< bare extension name (mp3)   */
+   int  i_len;			/**< psz_ext string length       */
+   int  i_types;		/**< mixed types                 */
+} fileext_t;			/**< single file extension       */
+
+typedef struct  {
+   vlc_mutex_t lock;		/**< lock to access this struct  */
+   int i_cnt;			/**< current number of extension */
+   int i_max;			/**< size of ext array           */
+   fileext_t * fileext;		/**< sorted array of extension   */
+} fileext_pool_t;
+
+static fileext_pool_t pool;
+
+static const char aud_exts[] = EXTENSIONS_AUDIO_STATIC;
+static const char vid_exts[] = EXTENSIONS_VIDEO_STATIC;
+static const char pls_exts[] = EXTENSIONS_PLAYLIST_STATIC;
+static const char sub_exts[] = EXTENSIONS_SUBTITLE_STATIC;
+
+static const char * name[FILEEXT_ALL+1] =
+{
+   "unknown",			/* 00 */
+   "audio",			/* 01 */
+   "video",			/* 02 */
+   "audio;video",		/* 03 */
+   "playlist",			/* 04 */
+   "audio;playlist",		/* 05 */
+   "playlist;video",		/* 06 */
+   "media",			/* 07 */
+   "subtitle",			/* 08 */
+   "audio;subtitle",		/* 09 */
+   "video;subtitle",		/* 0A */
+   "audio;video;subtitle",	/* 0B */
+   "playlist;subtitle",		/* 0C */
+   "audio;playlist;subtitle",	/* 0D */
+   "audio;video;subtitle",	/* 0E */
+   "media;subtitle",		/* 0F */
+   "all",			/* 10 */
+};
+
+static inline int valid_type(const int type)
+{
+   return ( type >= FILEEXT_UNKNOWN && type <= FILEEXT_ALL );
+}
+
+/**
+ * Check for a valid extension.
+ *
+ * \param  ext  extension to check "*.ext" or ".ext" or "ext"
+ * \retval   0  invalid
+ * \retval  >0  ext length (strlen(ext))
+ */
+static int valid_ext(const char * ext)
+{
+   int max = FILEEXT_LEN_MAX, len = 0;
+   /* basic checking: valid string starting by a dot '.' */
+   if( !ext ) return 0;
+   if( ext[len] == '*' ) ++len, ++max;
+   if( ext[len] == '.' ) ++len, ++max;
+   /* parsing: only valid character for extension are alphanum */
+   for(; ext[len]; ++len )
+      if ( len >= max || !isalnum( ext[len] )) return 0;
+   return len;
+}
+
+static int parse_exts(const char ** p_exts)
+{
+   const char * exts = *p_exts;
+   int l;
+
+   if ( *exts == '*' ) ++exts;	   /* ignore '*' prefix */
+   if ( *exts == '.' ) ++exts;	   /* ignore '.' prefix */
+   for ( l = 0; isalnum(exts[l]); ++l ) /* measure extension length */
+      ;
+
+   /* 1 <= length <= LEN_MAX */
+   if ( l<1 || l>FILEEXT_LEN_MAX )
+      return 0;
+   /* nul or ';' terminated  */
+   if ( exts[l] && exts[l] != ';' )
+      return 0;
+
+   *p_exts = exts;
+   return l;
+}
+
+static void pool_cleanup(fileext_pool_t * pool)
+{
+   pool->i_cnt  = 0;
+   pool->i_max  = 0;
+   free (pool->fileext);
+   pool->fileext = 0;
+   vlc_mutex_destroy(&pool->lock);
+}
+
+static int pool_init(fileext_pool_t * pool)
+{
+   memset(pool, 0, sizeof(*pool));
+   vlc_mutex_init(&pool->lock);
+   return VLC_SUCCESS;
+}
+
+static inline void pool_lock(fileext_pool_t * pool)
+{
+   vlc_mutex_lock(&pool->lock);
+}
+
+static inline void pool_unlock(fileext_pool_t * pool)
+{
+    vlc_mutex_unlock(&pool->lock);
+}
+
+static char * pool_get(fileext_pool_t * pool, int types)
+{
+   int l, i;
+   const int always = ( types == FILEEXT_ALL );
+   char * exts;
+
+   pool_lock(pool);
+
+   /* count matching extensions */
+   for( l = i = 0; i < pool->i_cnt; ++i )
+   {
+      if( always || pool->fileext[i].i_types == types ||
+	  (pool->fileext[i].i_types & types) )
+      {
+	 l += pool->fileext[i].i_len + 3;
+      }
+   }
+   /* malloc string */
+   exts = malloc( l );
+
+   if (exts)
+   {
+      for( l = i = 0; i < pool->i_cnt; ++i )
+      {
+	 if( always || pool->fileext[i].i_types == types ||
+	     (pool->fileext[i].i_types & types) )
+	 {
+	    exts[l++] = '*';
+	    exts[l++] = '.';
+	    strcpy(exts+l, pool->fileext[i].psz_ext);
+	    l += pool->fileext[i].i_len;
+	    exts[l++] = ';';
+	    }
+      }
+      if( l > 0 ) --l;
+      exts[l] = 0;
+   }
+   pool_unlock(pool);
+   return exts;
+}
+
+/* Copy buffer with lowercase transform */
+static inline void lowercpy(char * d, const char * s, const int l)
+{
+   int i;
+   for( i = 0; i < l; ++i )
+      d[i] = tolower(s[i]);
+}
+
+/* Compare 2 fileext_t (qsort and bsearch callback) */
+static int fileext_cmp(const void * p_va, const void * p_vb)
+{
+   const fileext_t * p_fe_a = p_va;
+   const fileext_t * p_fe_b = p_vb;
+   return strncmp(p_fe_a->psz_ext, p_fe_b->psz_ext, sizeof(p_fe_a->psz_ext));
+}
+
+/* MUST BE LOCK
+ * Ascending sort file extension pool.
+ */
+static void pool_sort(fileext_pool_t * pool)
+{
+   qsort(pool->fileext, pool->i_cnt, sizeof(fileext_t), fileext_cmp);
+}
+
+/* MUST BE LOCK
+ * Search for a file extension in the pool.
+ * retval  0  not found
+ * retval !0  matching file extension
+ */
+static fileext_t * pool_search(const fileext_pool_t * pool,
+			       const char * ext, const int len)
+{
+   fileext_t key;
+   lowercpy(key.psz_ext,ext,len);
+   key.psz_ext[len] = 0;
+   return
+      bsearch(&key, pool->fileext, pool->i_cnt, sizeof(key), fileext_cmp);
+}
+
+/* MUST BE LOCK
+ *
+ * Ensure the pool has enough room to store max file extension. It the
+ * pool does not have enough room it will be reallocated. The new size
+ * of the pool is max(size*2,max).
+ *
+ * retval  VLC_SUCCESS   on success
+ * retval  VLC_ENOMEM    on failure (realloc failed)
+ */
+static int pool_ensure(fileext_pool_t * pool, int max)
+{
+   if( max > pool->i_max )
+   {
+      fileext_t * fileext;
+      int newmax = pool->i_max * 2;
+
+      if (newmax < max) newmax = max;
+      fileext = realloc(pool->fileext, newmax * sizeof(fileext_t));
+      if( ! fileext )
+	 return VLC_ENOMEM;
+      pool->fileext = fileext;
+      pool->i_max   = newmax;
+   }
+   return VLC_SUCCESS;
+}
+
+/* MUST BE LOCK */
+static int pool_add_ext(fileext_pool_t * pool,
+				const char * ext, int len, int types)
+{
+   fileext_t * key
+      = pool_search(pool,ext,len);
+   int creation = !key;
+   if( creation )
+   {
+      if ( pool_ensure(pool, pool->i_cnt+1 ) == VLC_SUCCESS )
+      {
+	 key = pool->fileext + pool->i_cnt++;
+	 key->i_types = types;
+	 lowercpy(key->psz_ext, ext, len);
+	 key->psz_ext[len] = 0;
+	 key->i_len = len;
+      }
+   }
+
+   if( key )
+      key->i_types |= types;
+
+   return key ? creation : -1;
+}
+
+/* MUST BE LOCK */
+static int pool_add_exts(fileext_pool_t * pool,
+			 const char * exts, int types)
+{
+   int l, cnt = 0, creation = 0;
+
+   if ( types == FILEEXT_ALL ) types = FILEEXT_ALL_KNOWN;
+   while( l = parse_exts(&exts), l )
+   {
+      int res = pool_add_ext(pool,exts,l,types);
+      if( res == -1 )
+      {
+	 l = 0; break;
+      }
+      ++cnt;
+      creation |= (res == 1);
+
+      /* ignore semicolons ';' */
+      for( exts += l; *exts == ';'; ++exts )
+	 ;
+      /* no more ? */
+      if( !*exts ) break;
+   }
+   if( creation ) pool_sort(pool);
+   return l ? cnt : -1;
+}
+
+/* MUST BE LOCKED
+ *
+ *  Currently only remove types; the extension is never deleted but
+ *  stay in the pool in the unknown category.
+ *
+ *  retval  -1  error
+ *  return  >0  number of extension affected
+ */
+static int pool_del_exts(fileext_pool_t * pool,
+			 const char * exts, int types)
+{
+   int l, cnt = 0;
+   const int mask = ( types == FILEEXT_ALL ) ? 0 : ~types;
+
+   while( l = parse_exts(&exts), l )
+   {
+      fileext_t * ext;
+      if (ext = pool_search(pool,exts,l), ext)
+      {
+	 ++cnt;
+	 ext->i_types &= mask;
+      }
+
+      /* ignore semicolons ';' */
+      for( exts += l; *exts == ';'; ++exts )
+	 ;
+      /* no more ? */
+      if( !*exts ) break;
+   }
+   return l ? cnt : -1;
+}
+
+void fileext_Cleanup(void)
+{
+   pool_cleanup(&pool);
+}
+
+/**
+ * Initialise extension manager.
+ *
+ * \retval  VLC_SUCCESS  on success
+ * \retval !VLC_SUCCESS  error code of failure
+ */
+int fileext_Init(void)
+{
+   int err;
+
+   err = pool_init(&pool) != VLC_SUCCESS;
+   if( ! err )
+   {
+      pool_lock(&pool);
+      for ( ; ; ) {
+	 err = VLC_EGENERIC;
+	 if ( 0 > pool_add_exts(&pool,aud_exts,FILEEXT_AUDIO))    break;
+	 if ( 0 > pool_add_exts(&pool,vid_exts,FILEEXT_VIDEO))    break;
+	 if ( 0 > pool_add_exts(&pool,pls_exts,FILEEXT_PLAYLIST)) break;
+	 if ( 0 > pool_add_exts(&pool,sub_exts,FILEEXT_SUBTITLE)) break;
+	 err = VLC_SUCCESS;
+	 break;
+      }
+      pool_unlock(&pool);
+   }
+   return err;
+}
+
+/**
+ * Get name of extension type(s).
+ *
+ * \param   types  extension type mask
+ * \retval  0      an invalid type
+ * \retval  other  a semicolon ';' separated list of extension type
+ */
+const char * fileext_TypeName( int types )
+{
+/*    return types >= FILEEXT_UNKNOWN || type <= FILEEXT_ALL */
+   return valid_type(types)
+      ? name[types]
+      : "invalid"
+      ;
+}
+
+/**
+ * Get name of extension type(s).
+ *
+ * \param   types  extension type mask
+ * \retval  0      an invalid type
+ * \retval  other  a semicolon ';' separated list of extension type
+ */
+int fileext_TypeOf( const char * ext )
+{
+   fileext_t * fileext;
+   int type = FILEEXT_ERROR;
+   int len  = valid_ext( ext );
+   if( !len )
+      return type;
+
+   pool_lock(&pool);
+   fileext = pool_search(&pool, ext, len);
+   type = fileext
+      ? fileext->i_types
+      : FILEEXT_UNKNOWN
+      ;
+   pool_unlock(&pool);
+
+   return type;
+}
+
+/**
+ * Get extension list of given types.
+ *
+ * \param   types  extension type mask
+ * \retval  0      an invalid type
+ * \retval  ""     an empty list
+ * \retval  other  a list of extension (e.g "*.ext1;*.ext2")
+ */
+char * fileext_Get( int types )
+{
+   return valid_type(types)
+      ? pool_get(&pool, types)
+      : 0
+      ;
+}
+
+/**
+ * Add a list of extension of given type.
+ *
+ * \param   psz_exts  a semicolon ';' separated list of extension
+ * \param   types     extension type mask (for a single type)
+ * \retval  VLC_EGENERIC  on failure
+ * \retval  VLC_SUCCESS   on success
+ */
+int fileext_Add( const char * psz_extlist, int types )
+{
+   int res;
+
+   pool_lock(&pool);
+   res = pool_add_exts(&pool, psz_extlist, types) >= 0
+     ? VLC_SUCCESS
+     : VLC_EGENERIC
+     ;
+   pool_unlock(&pool);
+
+   return res;
+}
+
+/**
+ * Delete (remove) a list of extension.
+ *
+ * \param   psz_exts  a semicolon ';' separated list of extension
+ * \param   types     extension type mask (for a single type)
+ * \retval  VLC_EGENERIC  on failure
+ * \retval  VLC_SUCCESS   on success
+ */
+int fileext_Del( const char * psz_extlist, int types )
+{
+   int res;
+
+   pool_lock(&pool);
+   res = pool_del_exts(&pool, psz_extlist, types) >= 0
+     ? VLC_SUCCESS
+     : VLC_EGENERIC
+     ;
+   pool_unlock(&pool);
+
+   return res;
+}
-- 
1.6.1.3

